# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
from .timespan import Timespan
import pydantic
from ...base.types.timestamp import Timestamp
import datetime as dt
from .kb_article_install_state_id import KbArticleInstallStateId
from .os import Os
from .product import Product
from ...base.types.url_string import UrlString
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class KbArticle(UncheckedBaseModel):
    """
    The KB Article object contains metadata that describes the patch or update.
    """

    avg_timespan: typing.Optional[Timespan] = pydantic.Field(default=None)
    """
    The average time to patch.
    """

    bulletin: typing.Optional[str] = pydantic.Field(default=None)
    """
    The kb article bulletin identifier.
    """

    classification: typing.Optional[str] = pydantic.Field(default=None)
    """
    The vendors classification of the kb article.
    """

    created_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The date the kb article was released by the vendor.
    """

    created_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The date the kb article was released by the vendor.
    """

    install_state: typing.Optional[str] = pydantic.Field(default=None)
    """
    The install state of the kb article.
    """

    install_state_id: typing.Optional[KbArticleInstallStateId] = pydantic.Field(
        default=None
    )
    """
    The normalized install state ID of the kb article.
    """

    is_superseded: typing.Optional[bool] = pydantic.Field(default=None)
    """
    The kb article has been replaced by another.
    """

    os: typing.Optional[Os] = pydantic.Field(default=None)
    """
    The operating system the kb article applies.
    """

    product: typing.Optional[Product] = pydantic.Field(default=None)
    """
    The product details the kb article applies.
    """

    severity: typing.Optional[str] = pydantic.Field(default=None)
    """
    The severity of the kb article.
    """

    size: typing.Optional[int] = pydantic.Field(default=None)
    """
    The size in bytes for the kb article.
    """

    src_url: typing.Optional[UrlString] = pydantic.Field(default=None)
    """
    The kb article link from the source vendor.
    """

    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    The title of the kb article.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier for the kb article.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import typing
import datetime as dt
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class NucleusFindingState(UncheckedBaseModel):
    due_date: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Due date for the finding.
    """

    comment: typing.Optional[str] = pydantic.Field(default=None)
    """
    Comment to add to the finding.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...common.types.base import Base
from .mapping_id import MappingId
import pydantic
from ...organization_base.types.organization_id import OrganizationId
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class MappingBase(Base):
    id: MappingId
    fullname: str = pydantic.Field()
    """
    Human friendly display name for this mapping.
    """

    organization_id: OrganizationId = pydantic.Field()
    """
    Organization that manages this Mapping.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
from .assessment import Assessment
import pydantic
from .check import Check
from .kb_article import KbArticle
from .key_value_object import KeyValueObject
from .compliance_status_id import ComplianceStatusId
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Compliance(UncheckedBaseModel):
    """
    The Compliance object contains information about Industry and Regulatory Framework standards, controls and requirements or details about custom assessments utilized in a compliance evaluation. Standards define broad security frameworks, controls represent specific security requirements within those frameworks, and checks are the testable verification points used to determine if controls are properly implemented.
    """

    assessments: typing.Optional[typing.List[Assessment]] = pydantic.Field(default=None)
    """
    A list of assessments associated with the compliance requirements evaluation.
    """

    category: typing.Optional[str] = pydantic.Field(default=None)
    """
    The category a control framework pertains to, as reported by the source tool, such as <code>Asset Management</code> or <code>Risk Assessment</code>.
    """

    checks: typing.Optional[typing.List[Check]] = pydantic.Field(default=None)
    """
    A list of compliance checks associated with specific industry standards or frameworks. Each check represents an individual rule or requirement that has been evaluated against a target device. Checks typically include details such as the check name (e.g., CIS: 'Ensure mounting of cramfs filesystems is disabled' or DISA STIG descriptive titles), unique identifiers (such as CIS identifier '1.1.1.1' or DISA STIG identifier 'V-230234'), descriptions (detailed explanations of security requirements or vulnerability discussions), and version information.
    """

    compliance_references: typing.Optional[typing.List[KbArticle]] = pydantic.Field(
        default=None
    )
    """
    A list of reference KB articles that provide information to help organizations understand, interpret, and implement compliance standards. They provide guidance, best practices, and examples.
    """

    compliance_standards: typing.Optional[typing.List[KbArticle]] = pydantic.Field(
        default=None
    )
    """
    A list of established guidelines or criteria that define specific requirements an organization must follow.
    """

    control: typing.Optional[str] = pydantic.Field(default=None)
    """
    A Control is a prescriptive, actionable set of specifications that strengthens device posture. The control specifies required security measures, while the specific implementation values are defined in control_parameters. E.g., CIS AWS Foundations Benchmark 1.2.0 - Control 2.1 - Ensure CloudTrail is enabled in all regions
    """

    control_parameters: typing.Optional[typing.List[KeyValueObject]] = pydantic.Field(
        default=None
    )
    """
    The list of control parameters evaluated in a Compliance check. E.g., parameters for CloudTrail configuration might include <code>multiRegionTrailEnabled: true</code>, <code>logFileValidationEnabled: true</code>, and <code>requiredRegions: [us-east-1, us-west-2]</code>
    """

    desc: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description or criteria of a control.
    """

    requirements: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The specific compliance requirements being evaluated. E.g., <code>PCI DSS Requirement 8.2.3 - Passwords must meet minimum complexity requirements</code> or <code>HIPAA Security Rule 164.312(a)(2)(iv) - Implement encryption and decryption mechanisms</code>
    """

    standards: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The regulatory or industry standards being evaluated for compliance.
    """

    status: typing.Optional[str] = pydantic.Field(default=None)
    """
    The resultant status of the compliance check normalized to the caption of the <code>status_id</code> value. In the case of 'Other', it is defined by the event source.
    """

    status_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    The resultant status code of the compliance check.
    """

    status_detail: typing.Optional[str] = pydantic.Field(default=None)
    """
    The contextual description of the <code>status, status_code</code> values.
    """

    status_details: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A list of contextual descriptions of the <code>status, status_code</code> values.
    """

    status_id: typing.Optional[ComplianceStatusId] = pydantic.Field(default=None)
    """
    The normalized status identifier of the compliance check.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .types import (
    CreateIntegrationWebHookRequest,
    CreateIntegrationWebHookResponse,
    IntegrationWebHook,
    IntegrationWebHookId,
    ListIntegrationWebHooksResponse,
)

__all__ = [
    "CreateIntegrationWebHookRequest",
    "CreateIntegrationWebHookResponse",
    "IntegrationWebHook",
    "IntegrationWebHookId",
    "ListIntegrationWebHooksResponse",
]



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .group import Group
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Policy(UncheckedBaseModel):
    """
    The Policy object describes the policies that are applicable. <p>Policy attributes provide traceability to the operational state of the security product at the time that the event was captured, facilitating forensics, troubleshooting, and policy tuning/adjustments.</p>
    """

    data: typing.Optional[typing.Optional[typing.Any]] = pydantic.Field(default=None)
    """
    Additional data about the policy such as the underlying JSON policy itself or other details.
    """

    desc: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the policy.
    """

    group: typing.Optional[Group] = pydantic.Field(default=None)
    """
    The policy group.
    """

    is_applied: typing.Optional[bool] = pydantic.Field(default=None)
    """
    A determination if the content of a policy was applied to a target or request, or not.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The policy name. For example: <code>IAM Policy</code>.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique identifier of the policy instance.
    """

    version: typing.Optional[str] = pydantic.Field(default=None)
    """
    The policy version number.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .account_type_id import AccountTypeId
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Account(UncheckedBaseModel):
    """
    The Account object contains details about the account that initiated or performed a specific activity within a system or application.
    """

    labels: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The list of labels/tags associated to the account.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the account (e.g. GCP Account Name).
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The account type, normalized to the caption of 'account_type_id'. In the case of 'Other', it is defined by the event source.
    """

    type_id: typing.Optional[AccountTypeId] = pydantic.Field(default=None)
    """
    The normalized account type identifier.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the account (e.g. AWS Account ID).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import typing
from ...member_base.types.member import Member
from .organization import Organization
from ...token_base.types.token_pair import TokenPair
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class CreateOrganizationResponseResult(UncheckedBaseModel):
    member: typing.Optional[Member] = None
    organization: Organization
    token: TokenPair

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class OrderDirection(str, enum.Enum):
    ASCENDING = "asc"
    """
    Order in ascending order.
    """

    DESCENDING = "desc"
    """
    Order in descending order.
    """

    _UNKNOWN = "__ORDERDIRECTION_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "OrderDirection":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        ascending: typing.Callable[[], T_Result],
        descending: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is OrderDirection.ASCENDING:
            return ascending()
        if self is OrderDirection.DESCENDING:
            return descending()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .webhook_item import WebhookItem
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class WebhookConfig(UncheckedBaseModel):
    provider_key: typing.Optional[str] = pydantic.Field(default=None)
    """
    The key used by Synqly to verify incoming webhook payloads sent by the Provider. The format and requirements for this key is Provider specific:
    
    - **ServiceNow**: Webhook payload signing is not currently supported, so the integrity of incoming ServiceNow payloads cannot be verified.
    - **Jira**: Synqly does _not_ automatically configure Jira webhooks. A user with administrator privileges must configure the webhook, including the signing key. If this key is not specified, Synqly will _not_ validate incoming webhooks from Jira. It is strongly recommended that a key is specified and configured with Jira to ensure the integrity of incoming payloads.
    """

    items: typing.List[WebhookItem] = pydantic.Field()
    """
    List of webhooks for an integration. If the provider supports webhooks, they will be sent to the servers provided in this list.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import pydantic
import typing
from .custom_endpoint import CustomEndpoint
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class CustomSynqly(UncheckedBaseModel):
    """
    Configuration for Synqly Custom Provider.
    """

    custom_id: str = pydantic.Field()
    """
    Customer provider ID associated with this credential configuration
    """

    endpoints: typing.Dict[str, CustomEndpoint] = pydantic.Field()
    """
    Custom endpoints. Review the custom provider definition for the custom_id you supplied for the list of endpoints that must be defined. All endpoint in the provider definition are required.
    """

    tags: typing.Dict[str, str] = pydantic.Field()
    """
    Tags that customize how provider requests are made, such as supplying a URL path parameter or account ID. Review the custom provider definition for the custom_id you supplied for the list of tags that must be defined. All tags in the provider definition are required.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Node(UncheckedBaseModel):
    """
    Represents a node or a vertex in a graph structure.
    """

    data: typing.Optional[typing.Optional[typing.Any]] = pydantic.Field(default=None)
    """
    Additional data about the node stored as key-value pairs. Can include custom properties specific to the node.
    """

    desc: typing.Optional[str] = pydantic.Field(default=None)
    """
    A human-readable description of the node's purpose or meaning in the graph.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    A human-readable name or label for the node. Should be descriptive and unique within the graph context.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    Categorizes the node into a specific class or type. Useful for grouping and filtering nodes.
    """

    uid: str = pydantic.Field()
    """
    A unique string or numeric identifier that distinguishes this node from all others in the graph. Must be unique across all nodes.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ....core.unchecked_base_model import UncheckedBaseModel
import pydantic
import typing
from ....core.pydantic_utilities import IS_PYDANTIC_V2


class ExternalReference(UncheckedBaseModel):
    source_name: str = pydantic.Field()
    """
    The source_name property specifies the name of the source that provided the external reference.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description property provides a human-readable explanation of the external reference.
    """

    url: typing.Optional[str] = pydantic.Field(default=None)
    """
    The url property specifies the URL of the external reference.
    """

    hash: typing.Optional[str] = pydantic.Field(default=None)
    """
    The hash property specifies a hash value that uniquely identifies the external reference.
    """

    external_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The external_id property specifies an identifier used by the source of the external reference.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .provider_capabilities_base import ProviderCapabilitiesBase
import typing
from .connector_or_id import ConnectorOrId
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ProviderCapabilities(ProviderCapabilitiesBase):
    connector: typing.Optional[ConnectorOrId] = pydantic.Field(default=None)
    """
    Id of the Connector that the Provider implements – or, if
    `expand=connector` is set – the details of the Connector. NOTE:
    The current default behavior is to return the ID of the
    Connector. This field will soon only be populated when
    `expand=connector` is set in the APIs that support it. It is
    recommended you use `connector_id` for a stable reference to the
    Connector ID. This field will soon only be populated when
    `expand=connector` is set in the APIs that support it.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...common.types.base import Base
from .notification_id import NotificationId
import pydantic
import typing
from ...ticketing.types.priority import Priority
from .notification_status import NotificationStatus
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class Notification(Base):
    """
    Notification object
    """

    id: NotificationId
    summary: str = pydantic.Field()
    """
    Notification summary.
    """

    priority: typing.Optional[Priority] = pydantic.Field(default=None)
    """
    Notification priority
    """

    notification_status: typing.Optional[NotificationStatus] = pydantic.Field(
        default=None
    )
    """
    Notification status
    """

    project: typing.Optional[str] = pydantic.Field(default=None)
    """
    Notification project
    """

    status: typing.Optional[str] = pydantic.Field(default=None)
    """
    The current status of the notification.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Notification description.
    """

    issue_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The notification's type.
    """

    creator: typing.Optional[str] = pydantic.Field(default=None)
    """
    The user who created this notification.
    """

    assignee: typing.Optional[str] = pydantic.Field(default=None)
    """
    Who notification is assigned to.
    """

    contact: typing.Optional[str] = pydantic.Field(default=None)
    """
    The notification contact information.
    """

    tags: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Associate tags with Notification
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .mcp_token_scope import McpTokenScope
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class CreateMcpTokenRequest(UncheckedBaseModel):
    token_ttl: typing.Optional[str] = pydantic.Field(default=None)
    """
    Token time-to-live. If not provided, defaults to 1 hour. May be set to a maximum of 24 hours. Use the format "1h", "1m", "1s" for hours, minutes, and seconds respectively, e.g., "2h" for 2 hours.
    """

    scope: McpTokenScope = pydantic.Field()
    """
    Controls the tools that are available to the MCP server,
    and the APIs that can be accessed by the MCP server.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .types import (
    CreateSiemEventsResponse,
    Evidence,
    GetEvidenceResponse,
    GetInvestigationResponse,
    Investigation,
    LogProvider,
    PatchInvestigationRequest,
    QueryInvestigationResponse,
    QueryLogProvidersResponse,
    QuerySiemAlertsResponse,
    QuerySiemEventsResponse,
)

__all__ = [
    "CreateSiemEventsResponse",
    "Evidence",
    "GetEvidenceResponse",
    "GetInvestigationResponse",
    "Investigation",
    "LogProvider",
    "PatchInvestigationRequest",
    "QueryInvestigationResponse",
    "QueryLogProvidersResponse",
    "QuerySiemAlertsResponse",
    "QuerySiemEventsResponse",
]



# This file was auto-generated by Fern from our API Definition.

from ....v_1_3_0.compliancefinding.classes.types.compliance_finding import (
    ComplianceFinding as ocsf_v_1_3_0_compliancefinding_classes_types_compliance_finding_ComplianceFinding,
)

ComplianceFinding = (
    ocsf_v_1_3_0_compliancefinding_classes_types_compliance_finding_ComplianceFinding
)



# This file was auto-generated by Fern from our API Definition.

from .types import (
    Account,
    AccountTypeId,
    Actor,
    ActorActorTypeId,
    Advisory,
    AdvisoryInstallStateId,
    AffectedCode,
    AffectedPackage,
    AffectedPackageTypeId,
    Agent,
    AgentTypeId,
    Analytic,
    AnalyticTypeId,
    Api,
    Attack,
    AuthFactor,
    AuthFactorFactorTypeId,
    Authorization,
    AutonomousSystem,
    Certificate,
    Cloud,
    Container,
    Cve,
    Cvss,
    CvssDepth,
    Cwe,
    Database,
    DatabaseTypeId,
    Databucket,
    DatabucketTypeId,
    Device,
    DeviceHwInfo,
    DeviceHwInfoCpuArchitectureId,
    DeviceNetworkStatusId,
    DeviceRiskLevelId,
    DeviceTypeId,
    DigitalSignature,
    DigitalSignatureAlgorithmId,
    DigitalSignatureStateId,
    Display,
    DnsAnswer,
    DnsAnswerFlagIds,
    DomainContact,
    DomainContactTypeId,
    Email,
    EmailAuth,
    EncryptionDetails,
    EncryptionDetailsAlgorithmId,
    Enrichment,
    EnvironmentVariable,
    Epss,
    Extension,
    Feature,
    File,
    FileConfidentialityId,
    FileDriveTypeId,
    FileTypeId,
    Fingerprint,
    FingerprintAlgorithmId,
    FirewallRule,
    Group,
    HttpHeader,
    Idp,
    IdpStateId,
    Image,
    KbArticle,
    KbArticleInstallStateId,
    KeyValueObject,
    KeyboardInfo,
    KillChainPhase,
    KillChainPhasePhaseId,
    LdapPerson,
    Location,
    Logger,
    LongString,
    Malware,
    MalwareClassificationIds,
    Metadata,
    Metric,
    NetworkInterface,
    NetworkInterfaceTypeId,
    Object,
    Observable,
    ObservableTypeId,
    Organization,
    Os,
    OsTypeId,
    Osint,
    OsintConfidenceId,
    OsintTlp,
    OsintTypeId,
    Package,
    PackageTypeId,
    Policy,
    Process,
    ProcessEntity,
    ProcessIntegrityId,
    Product,
    Remediation,
    Reputation,
    ReputationScoreId,
    Request,
    ResourceDetails,
    Response,
    San,
    Scim,
    ScimAuthProtocolId,
    ScimStateId,
    Script,
    ScriptTypeId,
    Service,
    Session,
    Sso,
    SsoAuthProtocolId,
    SubTechnique,
    Table,
    Tactic,
    Technique,
    Timespan,
    TimespanTypeId,
    Url,
    UrlCategoryIds,
    User,
    UserMfaStatusId,
    UserRiskLevelId,
    UserTypeId,
    UserUserStatusId,
    Vulnerability,
    Whois,
    WhoisDnssecStatusId,
)

__all__ = [
    "Account",
    "AccountTypeId",
    "Actor",
    "ActorActorTypeId",
    "Advisory",
    "AdvisoryInstallStateId",
    "AffectedCode",
    "AffectedPackage",
    "AffectedPackageTypeId",
    "Agent",
    "AgentTypeId",
    "Analytic",
    "AnalyticTypeId",
    "Api",
    "Attack",
    "AuthFactor",
    "AuthFactorFactorTypeId",
    "Authorization",
    "AutonomousSystem",
    "Certificate",
    "Cloud",
    "Container",
    "Cve",
    "Cvss",
    "CvssDepth",
    "Cwe",
    "Database",
    "DatabaseTypeId",
    "Databucket",
    "DatabucketTypeId",
    "Device",
    "DeviceHwInfo",
    "DeviceHwInfoCpuArchitectureId",
    "DeviceNetworkStatusId",
    "DeviceRiskLevelId",
    "DeviceTypeId",
    "DigitalSignature",
    "DigitalSignatureAlgorithmId",
    "DigitalSignatureStateId",
    "Display",
    "DnsAnswer",
    "DnsAnswerFlagIds",
    "DomainContact",
    "DomainContactTypeId",
    "Email",
    "EmailAuth",
    "EncryptionDetails",
    "EncryptionDetailsAlgorithmId",
    "Enrichment",
    "EnvironmentVariable",
    "Epss",
    "Extension",
    "Feature",
    "File",
    "FileConfidentialityId",
    "FileDriveTypeId",
    "FileTypeId",
    "Fingerprint",
    "FingerprintAlgorithmId",
    "FirewallRule",
    "Group",
    "HttpHeader",
    "Idp",
    "IdpStateId",
    "Image",
    "KbArticle",
    "KbArticleInstallStateId",
    "KeyValueObject",
    "KeyboardInfo",
    "KillChainPhase",
    "KillChainPhasePhaseId",
    "LdapPerson",
    "Location",
    "Logger",
    "LongString",
    "Malware",
    "MalwareClassificationIds",
    "Metadata",
    "Metric",
    "NetworkInterface",
    "NetworkInterfaceTypeId",
    "Object",
    "Observable",
    "ObservableTypeId",
    "Organization",
    "Os",
    "OsTypeId",
    "Osint",
    "OsintConfidenceId",
    "OsintTlp",
    "OsintTypeId",
    "Package",
    "PackageTypeId",
    "Policy",
    "Process",
    "ProcessEntity",
    "ProcessIntegrityId",
    "Product",
    "Remediation",
    "Reputation",
    "ReputationScoreId",
    "Request",
    "ResourceDetails",
    "Response",
    "San",
    "Scim",
    "ScimAuthProtocolId",
    "ScimStateId",
    "Script",
    "ScriptTypeId",
    "Service",
    "Session",
    "Sso",
    "SsoAuthProtocolId",
    "SubTechnique",
    "Table",
    "Tactic",
    "Technique",
    "Timespan",
    "TimespanTypeId",
    "Url",
    "UrlCategoryIds",
    "User",
    "UserMfaStatusId",
    "UserRiskLevelId",
    "UserTypeId",
    "UserUserStatusId",
    "Vulnerability",
    "Whois",
    "WhoisDnssecStatusId",
]



# This file was auto-generated by Fern from our API Definition.

from .create_organization_request import CreateOrganizationRequest
from .create_organization_response import CreateOrganizationResponse
from .create_organization_response_result import CreateOrganizationResponseResult
from .environment import Environment
from .get_organization_response import GetOrganizationResponse
from .list_organization_response import ListOrganizationResponse
from .organization import Organization
from .organization_id import OrganizationId
from .organization_options import OrganizationOptions
from .organization_type import OrganizationType

__all__ = [
    "CreateOrganizationRequest",
    "CreateOrganizationResponse",
    "CreateOrganizationResponseResult",
    "Environment",
    "GetOrganizationResponse",
    "ListOrganizationResponse",
    "Organization",
    "OrganizationId",
    "OrganizationOptions",
    "OrganizationType",
]



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .url_category_ids import UrlCategoryIds
from ...base.types.hostname import Hostname
from ...base.types.port import Port
from ...base.types.url_string import UrlString
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Url(UncheckedBaseModel):
    """
    The Uniform Resource Locator(URL) object describes the characteristics of a URL. Defined in <a target='_blank' href='https://datatracker.ietf.org/doc/html/rfc1738'>RFC 1738</a> and by D3FEND <a target='_blank' href='https://d3fend.mitre.org/dao/artifact/d3f:URL/'>d3f:URL</a>.
    """

    categories: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The Website categorization names, as defined by <code>category_ids</code> enum values.
    """

    category_ids: typing.Optional[typing.List[UrlCategoryIds]] = pydantic.Field(
        default=None
    )
    """
    The Website categorization identifiers.
    """

    domain: typing.Optional[str] = pydantic.Field(default=None)
    """
    The domain portion of the URL. For example: <code>example.com</code> in <code>https://sub.example.com</code>.
    """

    hostname: typing.Optional[Hostname] = pydantic.Field(default=None)
    """
    The URL host as extracted from the URL. For example: <code>www.example.com</code> from <code>www.example.com/download/trouble</code>.
    """

    path: typing.Optional[str] = pydantic.Field(default=None)
    """
    The URL path as extracted from the URL. For example: <code>/download/trouble</code> from <code>www.example.com/download/trouble</code>.
    """

    port: typing.Optional[Port] = pydantic.Field(default=None)
    """
    The URL port. For example: <code>80</code>.
    """

    query_string: typing.Optional[str] = pydantic.Field(default=None)
    """
    The query portion of the URL. For example: the query portion of the URL <code>http://www.example.com/search?q=bad&sort=date</code> is <code>q=bad&sort=date</code>.
    """

    resource_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The context in which a resource was retrieved in a web request.
    """

    scheme: typing.Optional[str] = pydantic.Field(default=None)
    """
    The scheme portion of the URL. For example: <code>http</code>, <code>https</code>, <code>ftp</code>, or <code>sftp</code>.
    """

    subdomain: typing.Optional[str] = pydantic.Field(default=None)
    """
    The subdomain portion of the URL. For example: <code>sub</code> in <code>https://sub.example.com</code> or <code>sub2.sub1</code> in <code>https://sub2.sub1.example.com</code>.
    """

    url_string: typing.Optional[UrlString] = pydantic.Field(default=None)
    """
    The URL string. See RFC 1738. For example: <code>http://www.example.com/download/trouble.exe</code>. Note: The URL path should not populate the URL string.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ....v_1_5_0.applicationsecurityposturefinding.objects.types.application import (
    Application,
)

Ocsf150Application = Application



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import pydantic
from ...operation_base.types.operation_input import OperationInput
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class CreateOperationRequest(UncheckedBaseModel):
    operation: str = pydantic.Field()
    """
    Name of the operation that will be run for this operation.
    """

    input: OperationInput = pydantic.Field()
    """
    Input parameters to the operation that will be run for this operation.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
from .autonomous_system import AutonomousSystem
import pydantic
from ...base.types.timestamp import Timestamp
import datetime as dt
from .whois_dnssec_status_id import WhoisDnssecStatusId
from .domain_contact import DomainContact
from ...base.types.email_address import EmailAddress
from ...base.types.subnet import Subnet
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Whois(UncheckedBaseModel):
    """
    The resources of a WHOIS record for a given domain. This can include domain names, IP address blocks, autonomous system information, and/or contact and registration information for a domain.
    """

    autonomous_system: typing.Optional[AutonomousSystem] = pydantic.Field(default=None)
    """
    The autonomous system information associated with a domain.
    """

    created_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    When the domain was registered or WHOIS entry was created.
    """

    created_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    When the domain was registered or WHOIS entry was created.
    """

    dnssec_status: typing.Optional[str] = pydantic.Field(default=None)
    """
    The normalized value of dnssec_status_id.
    """

    dnssec_status_id: typing.Optional[WhoisDnssecStatusId] = pydantic.Field(
        default=None
    )
    """
    Describes the normalized status of DNS Security Extensions (DNSSEC) for a domain.
    """

    domain: typing.Optional[str] = pydantic.Field(default=None)
    """
    The domain name corresponding to the WHOIS record.
    """

    domain_contacts: typing.Optional[typing.List[DomainContact]] = pydantic.Field(
        default=None
    )
    """
    An array of <code>Domain Contact</code> objects.
    """

    email_addr: typing.Optional[EmailAddress] = pydantic.Field(default=None)
    """
    The email address for the registrar's abuse contact
    """

    last_seen_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    When the WHOIS record was last updated or seen at.
    """

    last_seen_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    When the WHOIS record was last updated or seen at.
    """

    name_servers: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A collection of name servers related to a domain registration or other record.
    """

    phone_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    The phone number for the registrar's abuse contact
    """

    registrar: typing.Optional[str] = pydantic.Field(default=None)
    """
    The domain registrar.
    """

    status: typing.Optional[str] = pydantic.Field(default=None)
    """
    The status of a domain and its ability to be transferred, e.g., <code>clientTransferProhibited</code>.
    """

    subdomains: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    An array of subdomain strings. Can be used to collect several subdomains such as those from Domain Generation Algorithms (DGAs).
    """

    subnet: typing.Optional[Subnet] = pydantic.Field(default=None)
    """
    The IP address block (CIDR) associated with a domain.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Feature(UncheckedBaseModel):
    """
    The Feature object provides information about the software product feature that generated a specific event. It encompasses details related to the capabilities, components, user interface (UI) design, and performance upgrades associated with the feature.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the feature.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the feature.
    """

    version: typing.Optional[str] = pydantic.Field(default=None)
    """
    The version of the feature.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
from .aws_sqs_credential import AwsSqsCredential
import pydantic
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class SinkAwsSqs(UncheckedBaseModel):
    """
    Configuration for Amazon Simple Queue Service (SQS).

    [Configuration guide](https://docs.synqly.com/guides/provider-configuration/aws-sqs-sink-setup)
    """

    credential: AwsSqsCredential = pydantic.Field()
    """
    AWS Access Keys with write access to the configured SQS queue.
    """

    region: typing.Optional[str] = pydantic.Field(default=None)
    """
    Overrides the default AWS region. If not present, the region will be inferred from the URL.
    """

    url: str = pydantic.Field()
    """
    URL of the SQS queue where events are sent.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
from .autonomous_system import AutonomousSystem
import pydantic
from ...base.types.timestamp import Timestamp
import datetime as dt
from .whois_dnssec_status_id import WhoisDnssecStatusId
from .domain_contact import DomainContact
from ...base.types.email_address import EmailAddress
from ...base.types.subnet import Subnet
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Whois(UncheckedBaseModel):
    """
    The resources of a WHOIS record for a given domain. This can include domain names, IP address blocks, autonomous system information, and/or contact and registration information for a domain.
    """

    autonomous_system: typing.Optional[AutonomousSystem] = pydantic.Field(default=None)
    """
    The autonomous system information associated with a domain.
    """

    created_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    When the domain was registered or WHOIS entry was created.
    """

    created_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    When the domain was registered or WHOIS entry was created.
    """

    dnssec_status: typing.Optional[str] = pydantic.Field(default=None)
    """
    The normalized value of dnssec_status_id.
    """

    dnssec_status_id: typing.Optional[WhoisDnssecStatusId] = pydantic.Field(
        default=None
    )
    """
    Describes the normalized status of DNS Security Extensions (DNSSEC) for a domain.
    """

    domain: typing.Optional[str] = pydantic.Field(default=None)
    """
    The domain name corresponding to the WHOIS record.
    """

    domain_contacts: typing.Optional[typing.List[DomainContact]] = pydantic.Field(
        default=None
    )
    """
    An array of <code>Domain Contact</code> objects.
    """

    email_addr: typing.Optional[EmailAddress] = pydantic.Field(default=None)
    """
    The email address for the registrar's abuse contact
    """

    isp: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the Internet Service Provider (ISP).
    """

    isp_org: typing.Optional[str] = pydantic.Field(default=None)
    """
    The organization name of the Internet Service Provider (ISP). This represents the parent organization or company that owns/operates the ISP. For example, Comcast Corporation would be the ISP org for Xfinity internet service. This attribute helps identify the ultimate provider when ISPs operate under different brand names.
    """

    last_seen_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    When the WHOIS record was last updated or seen at.
    """

    last_seen_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    When the WHOIS record was last updated or seen at.
    """

    name_servers: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A collection of name servers related to a domain registration or other record.
    """

    phone_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    The phone number for the registrar's abuse contact
    """

    registrar: typing.Optional[str] = pydantic.Field(default=None)
    """
    The domain registrar.
    """

    status: typing.Optional[str] = pydantic.Field(default=None)
    """
    The status of a domain and its ability to be transferred, e.g., <code>clientTransferProhibited</code>.
    """

    subdomains: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    An array of subdomain strings. Can be used to collect several subdomains such as those from Domain Generation Algorithms (DGAs).
    """

    subnet: typing.Optional[Subnet] = pydantic.Field(default=None)
    """
    The IP address block (CIDR) associated with a domain.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ....v_1_4_0.cloudresourcesinventoryinfo.classes.types.cloud_resources_inventory_info import (
    CloudResourcesInventoryInfo as ocsf_v_1_4_0_cloudresourcesinventoryinfo_classes_types_cloud_resources_inventory_info_CloudResourcesInventoryInfo,
)

CloudResourcesInventoryInfo = ocsf_v_1_4_0_cloudresourcesinventoryinfo_classes_types_cloud_resources_inventory_info_CloudResourcesInventoryInfo



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..operation_id_generated.types.operation_id import OperationId
from ..core.request_options import RequestOptions
from .types.query_custom_response import QueryCustomResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.unchecked_base_model import construct_type
from ..common.errors.bad_request_error import BadRequestError
from ..common.types.problem import Problem
from ..common.errors.unauthorized_error import UnauthorizedError
from ..common.errors.forbidden_error import ForbiddenError
from ..common.errors.not_found_error import NotFoundError
from ..common.errors.method_not_allowed_error import MethodNotAllowedError
from ..common.errors.conflict_error import ConflictError
from ..common.errors.unsupported_media_type_error import UnsupportedMediaTypeError
from ..common.errors.too_many_requests_error import TooManyRequestsError
from ..common.errors.internal_server_error import InternalServerError
from ..common.errors.not_implemented_error import NotImplementedError
from ..common.errors.bad_gateway_error import BadGatewayError
from ..common.errors.service_unavailable_error import ServiceUnavailableError
from ..common.errors.gateway_timeout_error import GatewayTimeoutError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from .types.get_custom_response import GetCustomResponse
from .types.patch_custom_request import PatchCustomRequest
from .types.create_custom_response import CreateCustomResponse
from .types.create_batch_custom_response import CreateBatchCustomResponse
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CustomClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def query(
        self,
        operation: OperationId,
        *,
        meta: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        order: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        include_raw_data: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> QueryCustomResponse:
        """
        Queries operation

        Parameters
        ----------
        operation : OperationId
            what `operation` to query

        meta : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Add metadata to the response by invoking meta functions. Documentation for meta functions is available at https://docs.synqly.com/api-reference/meta-functions. Not all meta function are available at every endpoint.

        cursor : typing.Optional[str]
            Cursor to use to retrieve the next page of results.

        limit : typing.Optional[int]
            Number of `operation` objects to return in this page. Defaults to 100.

        order : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Select a field to order the results by.

        filter : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter results by this query.

        include_raw_data : typing.Optional[bool]
            Include the raw data in the response. Defaults to `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        QueryCustomResponse

        Examples
        --------
        from synqly import SynqlyEngine
        from synqly.operation_id_generated import OperationId

        client = SynqlyEngine(
            token="YOUR_TOKEN",
        )
        client.custom.query(
            operation=OperationId.APPSEC_GET_APPLICATION_FINDING_DETAILS,
            meta="string",
            cursor="string",
            limit=1,
            order="string",
            filter="string",
            include_raw_data=True,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/custom/operations/query/{jsonable_encoder(operation)}",
            method="GET",
            params={
                "meta": meta,
                "cursor": cursor,
                "limit": limit,
                "order": order,
                "filter": filter,
                "include_raw_data": include_raw_data,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    QueryCustomResponse,
                    construct_type(
                        type_=QueryCustomResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self,
        operation: OperationId,
        id: str,
        *,
        meta: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        include_raw_data: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetCustomResponse:
        """
        Retrieves an Topic by ID.

        Parameters
        ----------
        operation : OperationId
            what `operation` to query

        id : str
            ID of the `operation` to retrieve.

        meta : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Add metadata to the response by invoking meta functions. Documentation for meta functions is available at https://docs.synqly.com/api-reference/meta-functions. Not all meta function are available at every endpoint.

        include_raw_data : typing.Optional[bool]
            Include the raw data in the response. Defaults to `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCustomResponse

        Examples
        --------
        from synqly import SynqlyEngine
        from synqly.operation_id_generated import OperationId

        client = SynqlyEngine(
            token="YOUR_TOKEN",
        )
        client.custom.get(
            operation=OperationId.APPSEC_GET_APPLICATION_FINDING_DETAILS,
            id="string",
            meta="string",
            include_raw_data=True,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/custom/operations/get/{jsonable_encoder(operation)}/{jsonable_encoder(id)}",
            method="GET",
            params={
                "meta": meta,
                "include_raw_data": include_raw_data,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetCustomResponse,
                    construct_type(
                        type_=GetCustomResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def patch(
        self,
        operation: OperationId,
        id: str,
        *,
        request: PatchCustomRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetCustomResponse:
        """
        Updates an `operation` by ID.

        Parameters
        ----------
        operation : OperationId
            what `operation` to query

        id : str
            ID of the `operation` to update.

        request : PatchCustomRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCustomResponse

        Examples
        --------
        from synqly import SynqlyEngine
        from synqly.common import PatchOp, PatchOperation
        from synqly.operation_id_generated import OperationId

        client = SynqlyEngine(
            token="YOUR_TOKEN",
        )
        client.custom.patch(
            operation=OperationId.APPSEC_GET_APPLICATION_FINDING_DETAILS,
            id="string",
            request=[
                PatchOperation(
                    op=PatchOp.ADD,
                    path="string",
                    from_="string",
                    value={"key": "value"},
                )
            ],
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/custom/operations/patch/{jsonable_encoder(operation)}/{jsonable_encoder(id)}",
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetCustomResponse,
                    construct_type(
                        type_=GetCustomResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self,
        operation: OperationId,
        id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Deletes the object matching `id` for the operation matching `operation`.

        Parameters
        ----------
        operation : OperationId
            what `operation` to delete.

        id : str
            ID of the `operation` to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from synqly import SynqlyEngine
        from synqly.operation_id_generated import OperationId

        client = SynqlyEngine(
            token="YOUR_TOKEN",
        )
        client.custom.delete(
            operation=OperationId.APPSEC_GET_APPLICATION_FINDING_DETAILS,
            id="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/custom/operations/delete/{jsonable_encoder(operation)}/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def post(
        self,
        operation: OperationId,
        *,
        request: typing.Dict[str, typing.Optional[typing.Any]],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateCustomResponse:
        """
        Writes an `operation` object to the Custom provider configured with the token used for authentication.

        Parameters
        ----------
        operation : OperationId
            what `operation` to post

        request : typing.Dict[str, typing.Optional[typing.Any]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateCustomResponse

        Examples
        --------
        from synqly import SynqlyEngine
        from synqly.operation_id_generated import OperationId

        client = SynqlyEngine(
            token="YOUR_TOKEN",
        )
        client.custom.post(
            operation=OperationId.APPSEC_GET_APPLICATION_FINDING_DETAILS,
            request={"string": {"key": "value"}},
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/custom/operations/post/{jsonable_encoder(operation)}",
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    CreateCustomResponse,
                    construct_type(
                        type_=CreateCustomResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def post_batch(
        self,
        operation: OperationId,
        *,
        request: typing.Sequence[typing.Dict[str, typing.Optional[typing.Any]]],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateBatchCustomResponse:
        """
        Writes a batch of `operation` objects to the Custom provider configured with the token used for authentication.

        Parameters
        ----------
        operation : OperationId
            what `operation` to post

        request : typing.Sequence[typing.Dict[str, typing.Optional[typing.Any]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateBatchCustomResponse

        Examples
        --------
        from synqly import SynqlyEngine
        from synqly.operation_id_generated import OperationId

        client = SynqlyEngine(
            token="YOUR_TOKEN",
        )
        client.custom.post_batch(
            operation=OperationId.APPSEC_GET_APPLICATION_FINDING_DETAILS,
            request=[{"string": {"key": "value"}}],
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/custom/operations/post-batch/{jsonable_encoder(operation)}",
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    CreateBatchCustomResponse,
                    construct_type(
                        type_=CreateBatchCustomResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncCustomClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def query(
        self,
        operation: OperationId,
        *,
        meta: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        order: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        include_raw_data: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> QueryCustomResponse:
        """
        Queries operation

        Parameters
        ----------
        operation : OperationId
            what `operation` to query

        meta : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Add metadata to the response by invoking meta functions. Documentation for meta functions is available at https://docs.synqly.com/api-reference/meta-functions. Not all meta function are available at every endpoint.

        cursor : typing.Optional[str]
            Cursor to use to retrieve the next page of results.

        limit : typing.Optional[int]
            Number of `operation` objects to return in this page. Defaults to 100.

        order : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Select a field to order the results by.

        filter : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter results by this query.

        include_raw_data : typing.Optional[bool]
            Include the raw data in the response. Defaults to `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        QueryCustomResponse

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyEngine
        from synqly.operation_id_generated import OperationId

        client = AsyncSynqlyEngine(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.custom.query(
                operation=OperationId.APPSEC_GET_APPLICATION_FINDING_DETAILS,
                meta="string",
                cursor="string",
                limit=1,
                order="string",
                filter="string",
                include_raw_data=True,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/custom/operations/query/{jsonable_encoder(operation)}",
            method="GET",
            params={
                "meta": meta,
                "cursor": cursor,
                "limit": limit,
                "order": order,
                "filter": filter,
                "include_raw_data": include_raw_data,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    QueryCustomResponse,
                    construct_type(
                        type_=QueryCustomResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self,
        operation: OperationId,
        id: str,
        *,
        meta: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        include_raw_data: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetCustomResponse:
        """
        Retrieves an Topic by ID.

        Parameters
        ----------
        operation : OperationId
            what `operation` to query

        id : str
            ID of the `operation` to retrieve.

        meta : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Add metadata to the response by invoking meta functions. Documentation for meta functions is available at https://docs.synqly.com/api-reference/meta-functions. Not all meta function are available at every endpoint.

        include_raw_data : typing.Optional[bool]
            Include the raw data in the response. Defaults to `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCustomResponse

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyEngine
        from synqly.operation_id_generated import OperationId

        client = AsyncSynqlyEngine(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.custom.get(
                operation=OperationId.APPSEC_GET_APPLICATION_FINDING_DETAILS,
                id="string",
                meta="string",
                include_raw_data=True,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/custom/operations/get/{jsonable_encoder(operation)}/{jsonable_encoder(id)}",
            method="GET",
            params={
                "meta": meta,
                "include_raw_data": include_raw_data,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetCustomResponse,
                    construct_type(
                        type_=GetCustomResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def patch(
        self,
        operation: OperationId,
        id: str,
        *,
        request: PatchCustomRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetCustomResponse:
        """
        Updates an `operation` by ID.

        Parameters
        ----------
        operation : OperationId
            what `operation` to query

        id : str
            ID of the `operation` to update.

        request : PatchCustomRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCustomResponse

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyEngine
        from synqly.common import PatchOp, PatchOperation
        from synqly.operation_id_generated import OperationId

        client = AsyncSynqlyEngine(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.custom.patch(
                operation=OperationId.APPSEC_GET_APPLICATION_FINDING_DETAILS,
                id="string",
                request=[
                    PatchOperation(
                        op=PatchOp.ADD,
                        path="string",
                        from_="string",
                        value={"key": "value"},
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/custom/operations/patch/{jsonable_encoder(operation)}/{jsonable_encoder(id)}",
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetCustomResponse,
                    construct_type(
                        type_=GetCustomResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self,
        operation: OperationId,
        id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Deletes the object matching `id` for the operation matching `operation`.

        Parameters
        ----------
        operation : OperationId
            what `operation` to delete.

        id : str
            ID of the `operation` to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyEngine
        from synqly.operation_id_generated import OperationId

        client = AsyncSynqlyEngine(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.custom.delete(
                operation=OperationId.APPSEC_GET_APPLICATION_FINDING_DETAILS,
                id="string",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/custom/operations/delete/{jsonable_encoder(operation)}/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def post(
        self,
        operation: OperationId,
        *,
        request: typing.Dict[str, typing.Optional[typing.Any]],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateCustomResponse:
        """
        Writes an `operation` object to the Custom provider configured with the token used for authentication.

        Parameters
        ----------
        operation : OperationId
            what `operation` to post

        request : typing.Dict[str, typing.Optional[typing.Any]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateCustomResponse

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyEngine
        from synqly.operation_id_generated import OperationId

        client = AsyncSynqlyEngine(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.custom.post(
                operation=OperationId.APPSEC_GET_APPLICATION_FINDING_DETAILS,
                request={"string": {"key": "value"}},
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/custom/operations/post/{jsonable_encoder(operation)}",
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    CreateCustomResponse,
                    construct_type(
                        type_=CreateCustomResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def post_batch(
        self,
        operation: OperationId,
        *,
        request: typing.Sequence[typing.Dict[str, typing.Optional[typing.Any]]],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateBatchCustomResponse:
        """
        Writes a batch of `operation` objects to the Custom provider configured with the token used for authentication.

        Parameters
        ----------
        operation : OperationId
            what `operation` to post

        request : typing.Sequence[typing.Dict[str, typing.Optional[typing.Any]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateBatchCustomResponse

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyEngine
        from synqly.operation_id_generated import OperationId

        client = AsyncSynqlyEngine(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.custom.post_batch(
                operation=OperationId.APPSEC_GET_APPLICATION_FINDING_DETAILS,
                request=[{"string": {"key": "value"}}],
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/custom/operations/post-batch/{jsonable_encoder(operation)}",
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    CreateBatchCustomResponse,
                    construct_type(
                        type_=CreateBatchCustomResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
from .service_now_credential import ServiceNowCredential
import typing
from .custom_field_mapping import CustomFieldMapping
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class TicketingServiceNowSir(UncheckedBaseModel):
    """
    Configuration for ServiceNow Security Incident Response (SIR).

    [Configuration guide](https://docs.synqly.com/guides/provider-configuration/servicenow-ticketing-setup)
    """

    credential: ServiceNowCredential
    custom_field_mappings: typing.Optional[typing.List[CustomFieldMapping]] = (
        pydantic.Field(default=None)
    )
    """
    Custom field mappings for this provider.
    """

    default_creator: typing.Optional[str] = pydantic.Field(default=None)
    """
    Default Creator (email) for the integration. If provided, the creator field becomes optional in ticket creation requests.
    """

    default_issue_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    Default Issue Type for the integration. If provided, the issue_type field becomes optional in ticket creation requests.
    """

    default_project: typing.Optional[str] = pydantic.Field(default=None)
    """
    Default Project for the integration. This maps to the custom table for tickets. This table should be derived from Security Incident table. Defaults to the security incident table if not specified.
    """

    url: str = pydantic.Field()
    """
    Base URL for the ServiceNow API.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
from .activity_id import ActivityId
import pydantic
import typing
from ...objects.types.actor import Actor
from ...objects.types.api import Api
from .category_uid import CategoryUid
from .class_uid import ClassUid
from ...objects.types.cloud import Cloud
from .confidence_id import ConfidenceId
from ...objects.types.object import Object
from ...objects.types.device import Device
from ...base.types.timestamp import Timestamp
import datetime as dt
from ...objects.types.enrichment import Enrichment
from ...objects.types.finding_info import FindingInfo
from ...objects.types.metadata import Metadata
from ...objects.types.observable import Observable
from ...objects.types.osint import Osint
from ...objects.types.resource_details import ResourceDetails
from .severity_id import SeverityId
from .status_id import StatusId
from .type_uid import TypeUid
from ...objects.types.vulnerability import Vulnerability
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class VulnerabilityFinding(UncheckedBaseModel):
    """
    The Vulnerability Finding event is a notification about weakness in an information system, system security procedures, internal controls, or implementation that could be exploited or triggered by a threat source.
    """

    activity_id: ActivityId = pydantic.Field()
    """
    The normalized identifier of the finding activity.
    """

    activity_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The finding activity name, as defined by the <code>activity_id</code>.
    """

    actor: typing.Optional[Actor] = pydantic.Field(default=None)
    """
    The actor object describes details about the user/role/process that was the source of the activity.
    """

    api: typing.Optional[Api] = pydantic.Field(default=None)
    """
    Describes details about a typical API (Application Programming Interface) call.
    """

    category_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The event category name, as defined by category_uid value: <code>Findings</code>.
    """

    category_uid: CategoryUid = pydantic.Field()
    """
    The category unique identifier of the event.
    """

    class_uid: ClassUid = pydantic.Field()
    """
    The unique identifier of a class. A class describes the attributes available in an event.
    """

    cloud: typing.Optional[Cloud] = pydantic.Field(default=None)
    """
    Describes details about the Cloud environment where the event was originally created or logged.
    """

    comment: typing.Optional[str] = pydantic.Field(default=None)
    """
    A user provided comment about the finding.
    """

    confidence: typing.Optional[str] = pydantic.Field(default=None)
    """
    The confidence, normalized to the caption of the confidence_id value. In the case of 'Other', it is defined by the event source.
    """

    confidence_id: typing.Optional[ConfidenceId] = pydantic.Field(default=None)
    """
    The normalized confidence refers to the accuracy of the rule that created the finding. A rule with a low confidence means that the finding scope is wide and may create finding reports that may not be malicious in nature.
    """

    confidence_score: typing.Optional[int] = pydantic.Field(default=None)
    """
    The confidence score as reported by the event source.
    """

    count: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of times that events in the same logical group occurred during the event <strong>Start Time</strong> to <strong>End Time</strong> period.
    """

    custom_fields: typing.Optional[Object] = pydantic.Field(default=None)
    """
    A list of custom fields
    """

    device: typing.Optional[Device] = pydantic.Field(default=None)
    """
    Describes the affected device/host. It can be used in conjunction with <code>Affected Resource(s)</code>. <p> e.g. Specific details about an AWS EC2 instance, that is affected by the Finding.</p>
    """

    duration: typing.Optional[int] = pydantic.Field(default=None)
    """
    The event duration or aggregate time, the amount of time the event covers from <code>start_time</code> to <code>end_time</code> in milliseconds.
    """

    end_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The time of the most recent event included in the finding.
    """

    end_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time of the most recent event included in the finding.
    """

    enrichments: typing.Optional[typing.List[Enrichment]] = pydantic.Field(default=None)
    """
    The additional information from an external data source, which is associated with the event or a finding. For example add location information for the IP address in the DNS answers:</p><code>[{"name": "answers.ip", "value": "92.24.47.250", "type": "location", "data": {"city": "Socotra", "continent": "Asia", "coordinates": [-25.4153, 17.0743], "country": "YE", "desc": "Yemen"}}]</code>
    """

    finding_info: FindingInfo = pydantic.Field()
    """
    Describes the supporting information about a generated finding.
    """

    message: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the event/finding, as defined by the source.
    """

    metadata: Metadata = pydantic.Field()
    """
    The metadata associated with the event or a finding.
    """

    observables: typing.Optional[typing.List[Observable]] = pydantic.Field(default=None)
    """
    The observables associated with the event or a finding.
    """

    osint: typing.Optional[typing.List[Osint]] = pydantic.Field(default=None)
    """
    The OSINT (Open Source Intelligence) object contains details related to an indicator such as the indicator itself, related indicators, geolocation, registrar information, subdomains, analyst commentary, and other contextual information. This information can be used to further enrich a detection or finding by providing decisioning support to other analysts and engineers.
    """

    raw_data: typing.Optional[str] = pydantic.Field(default=None)
    """
    The raw event/finding data as received from the source.
    """

    resource: typing.Optional[ResourceDetails] = pydantic.Field(default=None)
    """
    Describes details about the resource that is affected by the vulnerability/vulnerabilities.
    """

    resources: typing.Optional[typing.List[ResourceDetails]] = pydantic.Field(
        default=None
    )
    """
    Describes details about the resource/resources that are affected by the vulnerability/vulnerabilities.
    """

    severity: typing.Optional[str] = pydantic.Field(default=None)
    """
    The event/finding severity, normalized to the caption of the severity_id value. In the case of 'Other', it is defined by the source.
    """

    severity_id: SeverityId = pydantic.Field()
    """
    <p>The normalized identifier of the event/finding severity.</p>The normalized severity is a measurement the effort and expense required to manage and resolve an event or incident. Smaller numerical values represent lower impact events, and larger numerical values represent higher impact events.
    """

    start_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The time of the least recent event included in the finding.
    """

    start_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time of the least recent event included in the finding.
    """

    status: typing.Optional[str] = pydantic.Field(default=None)
    """
    The normalized status of the Finding set by the consumer normalized to the caption of the status_id value. In the case of 'Other', it is defined by the source.
    """

    status_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    The event status code, as reported by the event source.<br /><br />For example, in a Windows Failed Authentication event, this would be the value of 'Failure Code', e.g. 0x18.
    """

    status_detail: typing.Optional[str] = pydantic.Field(default=None)
    """
    The status detail contains additional information about the event/finding outcome.
    """

    status_id: typing.Optional[StatusId] = pydantic.Field(default=None)
    """
    The normalized status identifier of the Finding, set by the consumer.
    """

    time: Timestamp = pydantic.Field()
    """
    The normalized event occurrence time or the finding creation time.
    """

    time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The normalized event occurrence time or the finding creation time.
    """

    timezone_offset: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of minutes that the reported event <code>time</code> is ahead or behind UTC, in the range -1,080 to +1,080.
    """

    type_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The event/finding type name, as defined by the type_uid.
    """

    type_uid: TypeUid = pydantic.Field()
    """
    The event/finding type ID. It identifies the event's semantics and structure. The value is calculated by the logging system as: <code>class_uid \* 100 + activity_id</code>.
    """

    unmapped: typing.Optional[Object] = pydantic.Field(default=None)
    """
    The attributes that are not mapped to the event schema. The names and values of those attributes are specific to the event source.
    """

    vulnerabilities: typing.List[Vulnerability] = pydantic.Field()
    """
    This object describes vulnerabilities reported in a security finding.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .types import (
    ActivityId,
    CategoryUid,
    ClassUid,
    FileHosting,
    SeverityId,
    StatusId,
    TypeUid,
)

__all__ = [
    "ActivityId",
    "CategoryUid",
    "ClassUid",
    "FileHosting",
    "SeverityId",
    "StatusId",
    "TypeUid",
]



# This file was auto-generated by Fern from our API Definition.

from ....core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .marking_definition import MarkingDefinition
from ....core.pydantic_utilities import IS_PYDANTIC_V2


class GranularMarking(UncheckedBaseModel):
    lang: typing.Optional[str] = pydantic.Field(default=None)
    """
    The lang property specifies the language used in the properties of the marking_definition object.
    """

    marking_ref: typing.Optional[MarkingDefinition] = pydantic.Field(default=None)
    """
    The marking_ref property specifies the id property of the marking_definition object that applies to this object.
    """

    selectors: typing.List[str] = pydantic.Field()
    """
    The selectors property is an array of strings that represent the selectors that are used to apply the marking_definition object to this object.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...core.unchecked_base_model import UncheckedBaseModel
import typing
from .incident_response_team_union import IncidentResponseTeamUnion
from ...common.types.id import Id
from ...engine.types.object import Object
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic
from .incident_response_user_union import IncidentResponseUserUnion
import typing_extensions
from ...core.unchecked_base_model import UnionMetadata


class IncidentResponseEscalationTargetUnion_IncidentResponseUser(UncheckedBaseModel):
    type: typing.Literal["incident_response_user"] = "incident_response_user"
    email_addr: typing.Optional[str] = None
    teams: typing.Optional[typing.List[IncidentResponseTeamUnion]] = None
    name: typing.Optional[str] = None
    uid: Id
    timezone: typing.Optional[str] = None
    job_title: typing.Optional[str] = None
    src_url: typing.Optional[str] = None
    unmapped: typing.Optional[Object] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class IncidentResponseEscalationTargetUnion_IncidentResponseTeam(UncheckedBaseModel):
    type: typing.Literal["incident_response_team"] = "incident_response_team"
    desc: typing.Optional[str] = None
    name: typing.Optional[str] = None
    src_url: typing.Optional[str] = None
    uid: Id
    unmapped: typing.Optional[Object] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class IncidentResponseEscalationTargetUnion_IncidentResponseSchedule(
    UncheckedBaseModel
):
    type: typing.Literal["incident_response_schedule"] = "incident_response_schedule"
    desc: typing.Optional[str] = None
    name: typing.Optional[str] = None
    src_url: typing.Optional[str] = None
    uid: Id
    teams: typing.Optional[typing.List[IncidentResponseTeamUnion]] = None
    users: typing.Optional[typing.List[IncidentResponseUserUnion]] = None
    unmapped: typing.Optional[Object] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class IncidentResponseEscalationTargetUnion_IncidentResponseUserReference(
    UncheckedBaseModel
):
    type: typing.Literal["incident_response_user_reference"] = (
        "incident_response_user_reference"
    )
    name: typing.Optional[str] = None
    desc: typing.Optional[str] = None
    uid: Id
    src_url: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class IncidentResponseEscalationTargetUnion_IncidentResponseTeamReference(
    UncheckedBaseModel
):
    type: typing.Literal["incident_response_team_reference"] = (
        "incident_response_team_reference"
    )
    name: typing.Optional[str] = None
    desc: typing.Optional[str] = None
    uid: Id
    src_url: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class IncidentResponseEscalationTargetUnion_IncidentResponseScheduleReference(
    UncheckedBaseModel
):
    type: typing.Literal["incident_response_schedule_reference"] = (
        "incident_response_schedule_reference"
    )
    name: typing.Optional[str] = None
    desc: typing.Optional[str] = None
    uid: Id
    src_url: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


IncidentResponseEscalationTargetUnion = typing_extensions.Annotated[
    typing.Union[
        IncidentResponseEscalationTargetUnion_IncidentResponseUser,
        IncidentResponseEscalationTargetUnion_IncidentResponseTeam,
        IncidentResponseEscalationTargetUnion_IncidentResponseSchedule,
        IncidentResponseEscalationTargetUnion_IncidentResponseUserReference,
        IncidentResponseEscalationTargetUnion_IncidentResponseTeamReference,
        IncidentResponseEscalationTargetUnion_IncidentResponseScheduleReference,
    ],
    UnionMetadata(discriminant="type"),
]



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class CustomFieldMapping(UncheckedBaseModel):
    name: str = pydantic.Field()
    """
    Name for the custom field that you will use in the `custom_fields` field in the returned ticket objects.
    """

    project_id: str = pydantic.Field()
    """
    ID of the project this field mapping is associated with. ID of "\*" is used to apply to all projects.
    """

    provider_field_path: str = pydantic.Field()
    """
    Path to or name of the custom field in the provider.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
from ...account_base.types.account_id import AccountId
import pydantic
from ...integration_base.types.integration_id import IntegrationId
import datetime as dt
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class StatusEvent(UncheckedBaseModel):
    """
    Status event object
    """

    account_id: AccountId = pydantic.Field()
    """
    Account owner
    """

    integration_id: IntegrationId = pydantic.Field()
    """
    Integration object
    """

    created_at: dt.datetime = pydantic.Field()
    """
    Time created
    """

    status: str = pydantic.Field()
    """
    The current status of the notification.
    """

    error: typing.Optional[str] = pydantic.Field(default=None)
    """
    Error message
    """

    request: int = pydantic.Field()
    """
    Request number
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...engine.types.api_query_response import ApiQueryResponse
from ...engine.types.api_has_status import ApiHasStatus
import typing
from .log_provider import LogProvider
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class QueryLogProvidersResponse(ApiQueryResponse, ApiHasStatus):
    result: typing.List[LogProvider] = pydantic.Field()
    """
    List of available metadata.log_provider values available for querying events
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ....v_1_3_0.incidentfinding.classes.types.incident_finding import (
    IncidentFinding as ocsf_v_1_3_0_incidentfinding_classes_types_incident_finding_IncidentFinding,
)

IncidentFinding = (
    ocsf_v_1_3_0_incidentfinding_classes_types_incident_finding_IncidentFinding
)



# This file was auto-generated by Fern from our API Definition.

from ....v_1_6_0.httpactivity.classes.types.http_activity import (
    HttpActivity as ocsf_v_1_6_0_httpactivity_classes_types_http_activity_HttpActivity,
)

HttpActivity = ocsf_v_1_6_0_httpactivity_classes_types_http_activity_HttpActivity



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
from ...permissionset_base.types.permissions import Permissions
import typing
import pydantic
from .resource_restrictions import ResourceRestrictions
from .api_permission_map import ApiPermissionMap
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class PermissionSet(UncheckedBaseModel):
    name: Permissions
    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Description of when the permission set should be used and what permissions are granted by the permission set.
    """

    resource_restrictions: typing.Optional[typing.List[ResourceRestrictions]] = (
        pydantic.Field(default=None)
    )
    """
    Resources that can be used with this permission set
    """

    permissions: ApiPermissionMap = pydantic.Field()
    """
    API permissions granted by the permission set.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import typing
from .elasticsearch_auth_options import ElasticsearchAuthOptions
import pydantic
from .elasticsearch_credential import ElasticsearchCredential
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class SinkElasticsearch(UncheckedBaseModel):
    """
    Configuration for Elasticsearch.

    [Configuration guide](https://docs.synqly.com/guides/provider-configuration/elastic-setup)
    """

    auth_options: typing.Optional[ElasticsearchAuthOptions] = None
    create_index: str = pydantic.Field()
    """
    The index or data stream to use when writing events.
    """

    credential: ElasticsearchCredential
    skip_tls_verify: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When true, skips verification of the Elasticsearch TLS certificate.
    """

    url: str = pydantic.Field()
    """
    Base URL for the Elasticsearch API.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from .environment import SynqlyEngineEnvironment
import httpx
from .core.client_wrapper import SyncClientWrapper
from .appsec.client import AppsecClient
from .assets.client import AssetsClient
from .cloudsecurity.client import CloudsecurityClient
from .custom.client import CustomClient
from .edr.client import EdrClient
from .hooks.client import HooksClient
from .identity.client import IdentityClient
from .incidentresponse.client import IncidentresponseClient
from .integration_webhooks.client import IntegrationWebhooksClient
from .notifications.client import NotificationsClient
from .operations.client import OperationsClient
from .siem.client import SiemClient
from .sink.client import SinkClient
from .storage.client import StorageClient
from .ticketing.client import TicketingClient
from .vulnerabilities.client import VulnerabilitiesClient
from .core.client_wrapper import AsyncClientWrapper
from .appsec.client import AsyncAppsecClient
from .assets.client import AsyncAssetsClient
from .cloudsecurity.client import AsyncCloudsecurityClient
from .custom.client import AsyncCustomClient
from .edr.client import AsyncEdrClient
from .hooks.client import AsyncHooksClient
from .identity.client import AsyncIdentityClient
from .incidentresponse.client import AsyncIncidentresponseClient
from .integration_webhooks.client import AsyncIntegrationWebhooksClient
from .notifications.client import AsyncNotificationsClient
from .operations.client import AsyncOperationsClient
from .siem.client import AsyncSiemClient
from .sink.client import AsyncSinkClient
from .storage.client import AsyncStorageClient
from .ticketing.client import AsyncTicketingClient
from .vulnerabilities.client import AsyncVulnerabilitiesClient


class SynqlyEngine:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : SynqlyEngineEnvironment
        The environment to use for requests from the client. from .environment import SynqlyEngineEnvironment



        Defaults to SynqlyEngineEnvironment.SYNQLY



    token : typing.Union[str, typing.Callable[[], str]]
    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 300 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.Client]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from synqly import SynqlyEngine

    client = SynqlyEngine(
        token="YOUR_TOKEN",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: SynqlyEngineEnvironment = SynqlyEngineEnvironment.SYNQLY,
        token: typing.Union[str, typing.Callable[[], str]],
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 300 if httpx_client is None else None
        )
        self._client_wrapper = SyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            token=token,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.Client(
                timeout=_defaulted_timeout, follow_redirects=follow_redirects
            )
            if follow_redirects is not None
            else httpx.Client(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self.appsec = AppsecClient(client_wrapper=self._client_wrapper)
        self.assets = AssetsClient(client_wrapper=self._client_wrapper)
        self.cloudsecurity = CloudsecurityClient(client_wrapper=self._client_wrapper)
        self.custom = CustomClient(client_wrapper=self._client_wrapper)
        self.edr = EdrClient(client_wrapper=self._client_wrapper)
        self.hooks = HooksClient(client_wrapper=self._client_wrapper)
        self.identity = IdentityClient(client_wrapper=self._client_wrapper)
        self.incidentresponse = IncidentresponseClient(
            client_wrapper=self._client_wrapper
        )
        self.integration_webhooks = IntegrationWebhooksClient(
            client_wrapper=self._client_wrapper
        )
        self.notifications = NotificationsClient(client_wrapper=self._client_wrapper)
        self.operations = OperationsClient(client_wrapper=self._client_wrapper)
        self.siem = SiemClient(client_wrapper=self._client_wrapper)
        self.sink = SinkClient(client_wrapper=self._client_wrapper)
        self.storage = StorageClient(client_wrapper=self._client_wrapper)
        self.ticketing = TicketingClient(client_wrapper=self._client_wrapper)
        self.vulnerabilities = VulnerabilitiesClient(
            client_wrapper=self._client_wrapper
        )


class AsyncSynqlyEngine:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : SynqlyEngineEnvironment
        The environment to use for requests from the client. from .environment import SynqlyEngineEnvironment



        Defaults to SynqlyEngineEnvironment.SYNQLY



    token : typing.Union[str, typing.Callable[[], str]]
    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 300 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.AsyncClient]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from synqly import AsyncSynqlyEngine

    client = AsyncSynqlyEngine(
        token="YOUR_TOKEN",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: SynqlyEngineEnvironment = SynqlyEngineEnvironment.SYNQLY,
        token: typing.Union[str, typing.Callable[[], str]],
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 300 if httpx_client is None else None
        )
        self._client_wrapper = AsyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            token=token,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.AsyncClient(
                timeout=_defaulted_timeout, follow_redirects=follow_redirects
            )
            if follow_redirects is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self.appsec = AsyncAppsecClient(client_wrapper=self._client_wrapper)
        self.assets = AsyncAssetsClient(client_wrapper=self._client_wrapper)
        self.cloudsecurity = AsyncCloudsecurityClient(
            client_wrapper=self._client_wrapper
        )
        self.custom = AsyncCustomClient(client_wrapper=self._client_wrapper)
        self.edr = AsyncEdrClient(client_wrapper=self._client_wrapper)
        self.hooks = AsyncHooksClient(client_wrapper=self._client_wrapper)
        self.identity = AsyncIdentityClient(client_wrapper=self._client_wrapper)
        self.incidentresponse = AsyncIncidentresponseClient(
            client_wrapper=self._client_wrapper
        )
        self.integration_webhooks = AsyncIntegrationWebhooksClient(
            client_wrapper=self._client_wrapper
        )
        self.notifications = AsyncNotificationsClient(
            client_wrapper=self._client_wrapper
        )
        self.operations = AsyncOperationsClient(client_wrapper=self._client_wrapper)
        self.siem = AsyncSiemClient(client_wrapper=self._client_wrapper)
        self.sink = AsyncSinkClient(client_wrapper=self._client_wrapper)
        self.storage = AsyncStorageClient(client_wrapper=self._client_wrapper)
        self.ticketing = AsyncTicketingClient(client_wrapper=self._client_wrapper)
        self.vulnerabilities = AsyncVulnerabilitiesClient(
            client_wrapper=self._client_wrapper
        )


def _get_base_url(
    *, base_url: typing.Optional[str] = None, environment: SynqlyEngineEnvironment
) -> str:
    if base_url is not None:
        return base_url
    elif environment is not None:
        return environment.value
    else:
        raise Exception(
            "Please pass in either base_url or environment to construct the client"
        )



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import pydantic
import typing
from ...engine.types.object import Object
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class Project(UncheckedBaseModel):
    """
    Project in a ticketing system
    """

    id: str = pydantic.Field()
    """
    Unique identifier for this project. Use this id in the `project` field of a `Ticket`.
    """

    name: str = pydantic.Field()
    """
    Name of the project
    """

    custom_fields: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = (
        pydantic.Field(default=None)
    )
    """
    Custom mapped fields for this project.
    """

    unmapped: typing.Optional[Object] = pydantic.Field(default=None)
    """
    The attributes that are not mapped to the project schema. The names and values of those attributes are specific to the provider.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
from .aws_security_lake_credential import AwsSecurityLakeCredential
import pydantic
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class SinkAwsSecurityLake(UncheckedBaseModel):
    """
    Configuration for Amazon Security Lake as a Sink provider. Events are written directly to an AWS S3 bucket in Apache Parquet format.

    [Configuration guide](https://docs.synqly.com/guides/provider-configuration/amazon-security-lake-sink-setup)
    """

    credential: AwsSecurityLakeCredential = pydantic.Field()
    """
    AWS Access Keys with write access to the configured S3 bucket.
    """

    region: typing.Optional[str] = pydantic.Field(default=None)
    """
    Override the default AWS region for this integration. If not present, the region will be inferred from the URL.
    """

    url: str = pydantic.Field()
    """
    URL of the S3 bucket where the Amazon Security Lake events are stored.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .problem_details import ProblemDetails
import typing
from .problem_type import ProblemType
import datetime as dt
import pydantic
from .problem_cause import ProblemCause
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class Problem(ProblemDetails):
    """
    Generic base type for problems in Synqly. This object carries both machine-readable data as well as information intended for display purposes.
    """

    type: typing.Optional[ProblemType] = None
    occurred_at: dt.datetime = pydantic.Field()
    """
    The date and time the problem occurred.
    """

    status: int = pydantic.Field()
    """
    The HTTP status code of the problem. Matches the HTTP response code sent by the server.
    """

    instance: str = pydantic.Field()
    """
    A URI reference that identifies the specific occurrence of the problem. It may or may not yield further information if dereferenced.
    """

    cause: typing.Optional[typing.List[ProblemCause]] = pydantic.Field(default=None)
    """
    A list of the root cause(s) for this problem occurrence. Includes at minimum one root cause, and is otherwise an unordered list of causes.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .integration_ticket_when import IntegrationTicketWhen
from .ticketing_webhook_response import TicketingWebhookResponse

__all__ = ["IntegrationTicketWhen", "TicketingWebhookResponse"]



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class NetworkTraffic(UncheckedBaseModel):
    """
    The Network Traffic object describes characteristics of network traffic. Network traffic refers to data moving across a network at a given point of time.
    """

    bytes: typing.Optional[int] = pydantic.Field(default=None)
    """
    The total number of bytes (in and out).
    """

    bytes_in: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of bytes sent from the destination to the source.
    """

    bytes_missed: typing.Optional[int] = pydantic.Field(default=None)
    """
    Indicates the number of bytes missed, which is representative of packet loss.
    """

    bytes_out: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of bytes sent from the source to the destination.
    """

    chunks: typing.Optional[int] = pydantic.Field(default=None)
    """
    The total number of chunks (in and out).
    """

    chunks_in: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of chunks sent from the destination to the source.
    """

    chunks_out: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of chunks sent from the source to the destination.
    """

    packets: typing.Optional[int] = pydantic.Field(default=None)
    """
    The total number of packets (in and out).
    """

    packets_in: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of packets sent from the destination to the source.
    """

    packets_out: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of packets sent from the source to the destination.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .incident_response_team_union import IncidentResponseTeamUnion
from ...common.types.id import Id
from ...engine.types.object import Object
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class IncidentResponseUser(UncheckedBaseModel):
    email_addr: typing.Optional[str] = pydantic.Field(default=None)
    """
    The email address of the incident response user.
    """

    teams: typing.Optional[typing.List[IncidentResponseTeamUnion]] = pydantic.Field(
        default=None
    )
    """
    The incident response teams that the incident response user is a member of.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the incident response user.
    """

    uid: Id = pydantic.Field()
    """
    The unique identifier for the incident response user.
    """

    timezone: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timezone of the incident response user. This value is represented as a IANA timezone string.
    """

    job_title: typing.Optional[str] = pydantic.Field(default=None)
    """
    The job title of the incident response user.
    """

    src_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    A deep-linked url to the incident response user in a providers interface.
    """

    unmapped: typing.Optional[Object] = pydantic.Field(default=None)
    """
    The attributes that are not mapped to the incident response user schema. The names and values of those attributes are specific to each provider.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...core.unchecked_base_model import UncheckedBaseModel
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic
from ...credentials.types.o_auth_client_credential_id import OAuthClientCredentialId
import typing_extensions
from ...core.unchecked_base_model import UnionMetadata


class GoogleChronicleCredential_OAuthClient(UncheckedBaseModel):
    type: typing.Literal["o_auth_client"] = "o_auth_client"
    token_url: typing.Optional[str] = None
    client_id: str
    client_secret: str
    extra: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class GoogleChronicleCredential_OAuthClientId(UncheckedBaseModel):
    value: OAuthClientCredentialId
    type: typing.Literal["o_auth_client_id"] = "o_auth_client_id"

    class Config:
        smart_union = True


GoogleChronicleCredential = typing_extensions.Annotated[
    typing.Union[
        GoogleChronicleCredential_OAuthClient, GoogleChronicleCredential_OAuthClientId
    ],
    UnionMetadata(discriminant="type"),
]



# This file was auto-generated by Fern from our API Definition.

from .types import (
    BridgeGroup,
    BridgeGroupId,
    BridgeLocalConfig,
    BridgeLocalStats,
    BridgeStatus,
    CreateBridgeRequest,
    CreateBridgeResponse,
    CreateBridgeResponseResult,
    GetBridgeResponse,
    GetBridgeStatusResponse,
    ListBridgesResponse,
    PatchBridgeResponse,
    UpdateBridgeRequest,
    UpdateBridgeResponse,
)

__all__ = [
    "BridgeGroup",
    "BridgeGroupId",
    "BridgeLocalConfig",
    "BridgeLocalStats",
    "BridgeStatus",
    "CreateBridgeRequest",
    "CreateBridgeResponse",
    "CreateBridgeResponseResult",
    "GetBridgeResponse",
    "GetBridgeStatusResponse",
    "ListBridgesResponse",
    "PatchBridgeResponse",
    "UpdateBridgeRequest",
    "UpdateBridgeResponse",
]
